package SkeletonCart::App;

use strict;
use warnings;
use utf8;
use Carp;
use MT;
use SkeletonCart;

our @EXPORT = qw( is_mt42 is_mt43 is_mt5 is_mt51 is_mt52 is_mt6 is_mt5_or_later 
                    enabled stock_enabled install_dir
                    use_entry_id
                    use_item_option_field_1 use_item_option_field_2 use_item_option_field_3
                    use_item_option_field_4 use_item_option_field_5
                    item_option_number_1 item_option_number_2 item_option_number_3
                    item_option_number_4 item_option_number_5
                    item_option_label_1 item_option_label_2 item_option_label_3
                    item_option_label_4 item_option_label_5
                    use_item_weight use_dynamic_block
                    );

use base qw(Exporter);

use constant USE_SORTABLE_FIELD => 1;

sub is_mt42 {
    return (substr(MT->version_number, 0, 3) eq '4.2');
}

sub is_mt43 {
    return (substr(MT->version_number, 0, 3) eq '4.3');
}

sub is_mt5 {
    return (substr(MT->version_number, 0, 2) eq '5.');
}

sub is_mt51 {
    return (substr(MT->version_number, 0, 3) eq '5.1');
}

sub is_mt52 {
    return (substr(MT->version_number, 0, 3) eq '5.2');
}

sub is_mt6 {
    return (substr(MT->version_number, 0, 2) eq '6.');
}

sub is_mt5_or_later {
    return (substr(MT->version_number, 0, 1) >= 5);
}

sub is_mt7 {
    return (substr(MT->version_number, 0, 2) eq '7.');
}

sub is_mt6_or_later {
    return (substr(MT->version_number, 0, 1) >= 6);
}

sub is_mt7_or_later {
    return (substr(MT->version_number, 0, 1) >= 7);
}

sub config_template {
    my $plugin = shift;
    my $name = '';
    if(is_mt7_or_later()) {
        $name = 'config.tmpl';
    }
    else {
        $name = 'config.mt6.tmpl';
    }
    return $name;
}

sub _get_config_value {
    my ($key, $blog_id) = @_;
    return unless $key;
    if(!$blog_id) {
        my $app = MT->instance
            or return;
        return unless $app->can('blog');
        my $blog = $app->blog
            or return;
        $blog_id = $blog->id;
    }
    my $plugin = MT->component('SkeletonCartConnector');
    return $plugin->get_config_value($key, 'blog:' . $blog_id);
}

sub _get_func_name {
    my $name = (caller(1))[3];
    $name =~ s/.*:://;
    return $name;
}

sub enabled {
    return _get_config_value(_get_func_name(), @_);
}

sub stock_enabled {
    return _get_config_value(_get_func_name(), @_);
}

sub install_dir {
    return _get_config_value(_get_func_name(), @_);
}

sub use_entry_id {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_option_field_1 {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_option_field_2 {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_option_field_3 {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_option_field_4 {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_option_field_5 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_number_1 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_number_2 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_number_3 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_number_4 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_number_5 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_label_1 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_label_2 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_label_3 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_label_4 {
    return _get_config_value(_get_func_name(), @_);
}

sub item_option_label_5 {
    return _get_config_value(_get_func_name(), @_);
}

sub use_item_weight {
    return _get_config_value(_get_func_name(), @_);
}

sub use_dynamic_block {
    return _get_config_value(_get_func_name(), @_);
}

my @ITEMS = qw/id name price
               option_field_1 option_field_2 option_field_3
               option_field_4 option_field_5
               group_id
               quantity sales_status status_message weight/;

my %ITEMS = (
    id => {
        html => <<'HTML',
<input type="text" class="form-control short full" name="item_id" id="item_id" value="<mt:var name="item_id" escape="html">" mt:watch-change="1" autocomplete="off" /><br />
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="When there is not ItemID, the item information block is not displayed."></__trans_section></span>
HTML
        label => 'ItemID',
        condition => sub {
            !use_entry_id();
        },
    },
    name => {
        html => <<'HTML',
<input type="text" name="item_name" id="item_name" class="form-control full-width short full" value="<mt:var name="item_name" escape="html">" mt:watch-change="1" autocomplete="off" />
HTML
        label => 'ItemNAME',
    },
    price => {
        html => <<'HTML',
<input type="text" class="form-control short full" name="item_price" id="item_price" value="<mt:var name="item_price" escape="html">" mt:watch-change="1" autocomplete="off" />
HTML
        label => 'ItemPRICE',
    },
    option_field_1 => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = <<HTML;
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If you use Item Option, Please fill in one line with one value."></__trans_section></span><div><textarea rows="3" cols="18" class="form-control text full low full-width" name="item_option_field_1" id="item_option_field_1"><mt:var name="item_option_field_1"></textarea></div>
HTML
        },
        label => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            $plugin->get_config_value("item_option_label_1", 'blog:'.$blog_id );
        },
        condition => sub {
            use_item_option_field_1();
        },
    },
    option_field_2 => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = <<HTML;
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If you use Item Option, Please fill in one line with one value."></__trans_section></span><div><textarea rows="3" cols="18" class="form-control text full low full-width" name="item_option_field_2" id="item_option_field_2"><mt:var name="item_option_field_2"></textarea></div>
HTML
        },
        label => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            $plugin->get_config_value("item_option_label_2", 'blog:'.$blog_id );
        },
        condition => sub {
            use_item_option_field_2();
        },
    },
    option_field_3 => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = <<HTML;
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If you use Item Option, Please fill in one line with one value."></__trans_section></span><div><textarea rows="3" cols="18" class="form-control text full low full-width" name="item_option_field_3" id="item_option_field_3"><mt:var name="item_option_field_3"></textarea></div>
HTML
        },
        label => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            $plugin->get_config_value("item_option_label_3", 'blog:'.$blog_id );
        },
        condition => sub {
            use_item_option_field_3();
        },
    },
    option_field_4 => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = <<HTML;
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If you use Item Option, Please fill in one line with one value."></__trans_section></span><div><textarea rows="3" cols="18" class="form-control text full low full-width" name="item_option_field_4" id="item_option_field_4"><mt:var name="item_option_field_4"></textarea></div>
HTML
        },
        label => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            $plugin->get_config_value("item_option_label_4", 'blog:'.$blog_id );
        },
        condition => sub {
            use_item_option_field_4();
        },
    },
    option_field_5 => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = <<HTML;
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If you use Item Option, Please fill in one line with one value."></__trans_section></span><div><textarea rows="3" cols="18" class="form-control text full low full-width" name="item_option_field_5" id="item_option_field_5"><mt:var name="item_option_field_5"></textarea></div>
HTML
        },
        label => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            $plugin->get_config_value("item_option_label_5", 'blog:'.$blog_id );
        },
        condition => sub {
            use_item_option_field_5();
        },
    },
    group_id => {
        html => sub {
            my ($plugin, $app, $param, $tmpl) = @_;
            my $blog_id = $app->blog->id;
            my $innerHTML = '';
            foreach my $n (1..5) {
                my $label = $plugin->get_config_value("group_label_$n", 'blog:'.$blog_id )
                    or next;
                my $html = <<HTML;
<input type="radio" class="form-check-input" id="$n" name="item_group_id" value="$n"<mt:if name="item_group_id" eq="$n"> checked="checked"</mt:if> /><label class="form-check-label" for="$n">$label</label>
HTML
                $innerHTML .= $html;
            }
            $innerHTML = <<HTML;
<div class="form-check form-check-inline">
    $innerHTML
</div>
HTML
            return $innerHTML;
        },
        label => 'ItemGroupID',
    },
    quantity => {
        html => <<'HTML',
<__trans_section component="SkeletonCartConnector">
    <div class="form-row">
    <div class="col">
        <label for="item_stock_quantity"><__trans phrase="Real quantity" /></label>
        <input type="text" class="form-control text num" name="item_stock_quantity" id="item_stock_quantity" value="<mt:var name="item_stock_quantity" escape="html">" mt:watch-change="1" autocomplete="off" readonly="readonly" />
    </div>
    <div class="col">
        <label for="item_stock_quantity_adjust"><__trans phrase="Adjust quantity" /></label>
        <input type="text" class="form-control text num" name="item_stock_quantity_adjust" id="item_stock_quantity_adjust" value="<mt:var name="item_stock_quantity_adjust" escape="html" default="0" />" mt:watch-change="1" autocomplete="off" />
    </div>
    <div class="col-auto">
        <label for="item_stock_quantity_adjust_type">&nbsp;</label>
        <select name="item_stock_quantity_adjust_type" id="item_stock_quantity_adjust_type" class="form-control">
            <option value="add"><__trans phrase="Add" /></option>
            <option value="reduce"><__trans phrase="Reduce" /></option>
        </select><br />
    </div>

    </div>
    <span class="hint"><__trans phrase="Quantity of Item."></span>
</__trans_section>
HTML
        label => 'ItemStockQuantity',
        condition => sub {
            stock_enabled();
        },
    },
    sales_status => {
        html => <<'HTML',
<div class="form-check form-check-inline">
<input type="radio" class="form-check-input" id="on_sale" name="item_sales_status" value="1"<mt:if name="item_sales_status" eq="1"> checked="checked"</mt:if> /><label class="form-check-label" for="on_sale"> <__trans_section component="SkeletonCartConnector"><__trans phrase="On Sale"></__trans_section> </label>
<input type="radio" class="form-check-input" id="suspended" name="item_sales_status" value="0"<mt:unless name="item_sales_status"> checked="checked"</mt:unless> /><label class="form-check-label" for="suspended"> <__trans_section component="SkeletonCartConnector"><__trans phrase="Suspended"></__trans_section></label>
</div>
HTML
        label => 'ItemSalesStatus',
    },
    status_message => {
        html => <<'HTML',
<input type="text" name="item_status_message" id="item_status_message" class="form-control full-width full" value="<mt:if name="item_status_message"><mt:var name="item_status_message" escape="html"><mt:else><__trans_section component="SkeletonCartConnector"><__trans phrase="We are not in stock now."></__trans_section></mt:if>" mt:watch-change="1" autocomplete="off" /><br />
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="If out of stock, the message will be displayed instead of the buy button. Please refer to the bundled sample template."></__trans_section></span>
HTML
        label => 'Messages to be displayed during out-of-stock or sales pauses, such as next arrival information.',
    },
    weight => {
        html => <<'HTML',
<input type="text" class="form-control short full" name="item_weight" id="item_weight" value="<mt:var name="item_weight" escape="html">" mt:watch-change="1" autocomplete="off" /><br />
<span class="hint"><__trans_section component="SkeletonCartConnector"><__trans phrase="Enter item's weight."></__trans_section></span>
HTML
        label => 'ItemWeight',
        condition => sub {
            use_item_weight();
        },
    },
);

sub _get_html {
    my ($tmpl, $tmpl_str) = @_;
    require MT::Template;
    my $snip_tmpl = MT::Template->new(
        type   => 'scalarref',
        source => \$tmpl_str,
    ) or die 'Failed to create template object.';
    $snip_tmpl->param($tmpl->param);
    my $out = $snip_tmpl->output
        or die $snip_tmpl->errstr;
    my $plugin = MT->component('SkeletonCartConnector');
    $out = $plugin->translate_templatized($out);
    return $out;
}

sub _insert_element {
    my ($tmpl, $target, $id, $label, $innerHTML, $position, $field_loop, $prefs) = @_;
    my $plugin = MT->component('SkeletonCartConnector');
    $label = $plugin->translate($label);
    my $shown = $innerHTML ? 1 : 0;
    # if(USE_SORTABLE_FIELD && (is_mt51() || is_mt52() || is_mt6())) {
    if(USE_SORTABLE_FIELD && is_mt7_or_later()) {
        $innerHTML = _get_html($tmpl, $innerHTML);
        push @$field_loop, {
            field_html => $innerHTML,
            field_id => $id,
            field_name => $id,
            show_field => $shown,
            field_label => $label,
            label_class => 'top-label',
        };
        push @$prefs, $id unless grep { $_ eq $id } @$prefs;
    }
    else {
        my $host_node = $tmpl->getElementById($target);
        my $block_node = $tmpl->createElement(
            'app:setting', {
                id => $id,
                label => $label,
                label_class => 'top-label',
                shown => $shown,
            }
        );
        $block_node->innerHTML($innerHTML);
        if ($position eq 'after') {
            $tmpl->insertAfter($block_node, $host_node);
        } elsif ($position eq 'before') {
            $tmpl->insertBefore($block_node, $host_node);
        }
    }
}

sub _log {
    my ($param) = @_;
    require Data::Dumper;
    open my $f, ">>skeltoncart.log"
        or return;
    my $time = MT::Util::epoch2ts(undef, time) . ':';
    print $f $time;
    print $f Data::Dumper->Dump([$param]), "\n";
    close $f;
    return;
}

sub _code_to_value {
    my ($code, @args) = @_;
    my $value = $code;
    if(ref $code && ref $code eq 'CODE') {
        $value = $code->(@args);
    }
    return $value;
}

sub param_edit_entry {
    my ($eh, $app, $param, $tmpl) = @_;
    return unless enabled();
    return unless UNIVERSAL::isa($tmpl, 'MT::Template');

    require SkeletonCart::Item;
    my $entry_id = $param->{id};
    if($entry_id) {
        my ($item) = SkeletonCart::Item->load({ entry_id => $entry_id });
        if($item) {
            my $values = $item->get_values;
            foreach my $col (@{$item->column_names}) {
                if($col eq 'id' || $col eq 'entry_id') {
                    next;
                }
                elsif($col eq 'item_id') {
                    $param->{$col} = $values->{$col};
                }
                else {
                    $param->{'item_' . $col} = $values->{$col};
                }
            }
        }
    }

    # Set default group id
    if(!exists $param->{item_group_id}) {
        $param->{item_group_id} = 1;
    }

    my $plugin = MT->component('SkeletonCartConnector');
    my $field_loop = $param->{field_loop} || [];
    my $type = $app->param('_type') || 'entry';
    my $prefs_type = $type . '_prefs';
    my $default_prefs_type = "load_default_" .$prefs_type;
    my $prefs = $app->permissions->$prefs_type || $app->$default_prefs_type;
    my @fields;
    ($prefs) = split /\|/, $prefs;
    my @p = split /,/, $prefs;
    foreach my $p (@p) {
        if(grep { lc($p) eq $_ } qw(advanced default basic)) {
            push @fields, qw(title text category tags feedback publishing assets);
            if(lc($p) eq 'advanced') {
                push @fields, qw(excerpt feedback keywords)
            }
        }
        else {
            push @fields, $p;
        }
    }

    my $last_item = 'text';

    foreach my $item_name (@ITEMS) {
        my $item = $ITEMS{$item_name} or next;
        next if $item->{condition} && !$item->{condition}->();
        my $html = _code_to_value($item->{html}, $plugin, $app, $param, $tmpl);
        my $label = $plugin->translate(
            _code_to_value($item->{label}, $plugin, $app, $param, $tmpl));
        _insert_element($tmpl, $last_item, 'item_'.$item_name,
                        $label, $html, 'after', $field_loop, \@fields);
        $last_item = 'item_'.$item_name;
    }

    $param->{use_dynamic_block} = use_dynamic_block();
    $param->{stock_enabled} = stock_enabled();

    # if(USE_SORTABLE_FIELD && (is_mt51() || is_mt52() || is_mt6())) {
    if(USE_SORTABLE_FIELD && is_mt7_or_later()) {
        foreach my $sc_item (@ITEMS) {
            my $name = 'item_' . $sc_item;
            if(!grep { $_ eq $name } @fields) {
                push @fields, $name;
            }
        }
        my $i = 0;
        my %prefs = map { $_ => $i++ } @fields;
        $field_loop = [ sort {
            (defined $prefs{$a->{field_name}} ? $prefs{$a->{field_name}} : ~0)
                <=> (defined $prefs{$b->{field_name}} ? $prefs{$b->{field_name}} : ~0)
        } @$field_loop ];
        $param->{field_loop} = $field_loop;
    }

    return 1;
}

sub source_list_entry {
    my ($cb, $app, $tmpl_ref) = @_;
    return 1 if is_mt5_or_later();
    my $old = quotemeta(<<'HTML');
<$mt:include name="include/header.tmpl"$>
HTML
    my $new = <<HTML;
<mt:setvarblock name="html_head" append="1">
<style type="text/css">
.entry-listing-table .stock {
  white-space: nowrap;
  width: 1%;
}
</style>
</mt:setvarblock>

HTML
    $$tmpl_ref =~ s!($old)!$new$1!;
    return 1;
}

sub source_entry_table {
    return 1 unless stock_enabled();
    if(is_mt42() || is_mt43()) {
        source_entry_table_mt42(@_);
    }
    elsif(is_mt5_or_later()) {
        source_entry_table_mt5(@_);
    }
    return 1;
}

sub source_entry_table_mt42 {
    my ($cb, $app, $tmpl_ref) = @_;
    # thead
    my $old = quotemeta(<<'HTML');
                <th class="title"><__trans phrase="Title"></th>
HTML
    my $new = <<HTML;
                <th class="stock"><__trans_section component="SkeletonCartConnector"><__trans phrase="ItemStockQuantity"></__trans_section></th>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;
    # tbody
    $old = quotemeta(<<'HTML');
                <$mt:var name="title" escape="html"$>
            </mt:if>
        </mt:if>
                </td>
HTML
    $new = <<HTML;
<td class="stock">
  <mt:if name="has_edit_access">
    <mt:if name="is_power_edit">
      <input type="text" name="item_stock_quantity_<mt:var name="id"/>" value="<mt:var name="item_stock_quantity" escape="html" default="0" />" size="6" />
    <mt:else>
      <mt:var name="item_stock_quantity" escape="html" default="0" />
    </mt:if>
  <mt:else>
    <mt:var name="item_stock_quantity" escape="html" default="0" />
  </mt:if>
</td>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;
    return 1;
}

sub source_entry_table_mt5 {
    my ($cb, $app, $tmpl_ref) = @_;
    # thead
    my $old = quotemeta(<<'HTML');
                <th class="blog"><__trans phrase="Website/Blog"></th>
                </mt:if>
            </mt:unless>
        </mt:unless>
HTML
    my $new = <<HTML;
                <th class="stock"><__trans_section component="SkeletonCartConnector"><__trans phrase="ItemStockQuantity"></__trans_section></th>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;
    # tbody
    $old = quotemeta(<<'HTML');
                <$mt:var name="title" escape="html" default="..."$>
            </mt:if>
        </mt:if>
                </td>
HTML
    $new = <<HTML;
<td class="stock">
  <mt:if name="has_edit_access">
    <mt:if name="is_power_edit">
      <input type="text" name="item_stock_quantity_<mt:var name="id"/>" value="<mt:var name="item_stock_quantity" escape="html" default="0" />" size="6" readonly="readonly" />
      <input type="text" name="item_stock_quantity_adjust_<mt:var name="id"/>" value="<mt:var name="item_stock_quantity_adjust" escape="html" default="0" />" size="6" />
      <select name="item_stock_quantity_adjust_type_<mt:var name="id"/>">
        <option value="add"><__trans phrase="Add" /></option>
        <option value="reduce"><__trans phrase="Reduce" /></option>
      </select>
    <mt:else>
      <mt:var name="item_stock_quantity" escape="html" default="0" />
    </mt:if>
  <mt:else>
    <mt:var name="item_stock_quantity" escape="html" default="0" />
  </mt:if>
</td>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;
    return 1;
}

sub _normalize_number {
    my ($number) = @_;
    require Encode;
    require MT::I18N;
    my $enc = MT::I18N::guess_encoding($number);
    $number = Encode::decode($enc, $number)
        unless Encode::is_utf8($number);
    $number =~ tr/\x{ff10}-\x{ff19}\x{ff0c}\x{ff0e}\x{2212}/0-9,.\-/;
    $number =~ s/,//g;
    return $number;
}

sub pre_save_entry {
    my ($cb, $app, $entry, $org_entry) = @_;
    return 1 unless enabled() && (stock_enabled() || use_item_weight());
    my $id = $entry->id or return 1;
    my $perms = $app->permissions;
    my $user = $app->user;
        return 1 unless $perms->can_edit_entry($entry, $user, 1);
    my $plugin = MT->component('SkeletonCartConnector');

    # check quantity
    if(stock_enabled()) {
        my $quantity;
        if ($app->mode eq 'save_entries') {
            $quantity = _normalize_number($app->param('item_stock_quantity_'.$id)) || 0;
        } else {
            $quantity = _normalize_number($app->param('item_stock_quantity')) || 0;
        }
        return $cb->error(
            $plugin->translate('Stock Quantity must be a number.')
        ) unless $quantity =~ m!^[0-9]+$!;
        $entry->item_stock_quantity($quantity);
    }

    if($app->mode ne 'save_entries') {
        # check price
        my $price = _normalize_number($app->param('item_price'));
        return $cb->error(
            $plugin->translate('Price must be a number.')
        ) unless $price =~ m!^\-?[0-9]+$!;
        $entry->item_price($price);

        # check weight
        if(use_item_weight()) {
            my $weight = _normalize_number($app->param('item_weight'));
            if(defined $weight && length $weight > 0) {
                require Scalar::Util;
                if (!Scalar::Util::looks_like_number($weight) || $weight < 0) {
                    return $cb->error(
                        $plugin->translate('Item weight must be a positive number.')
                    );
                }
            }
            $entry->item_weight($weight);
        }
    }

    return 1;
}

sub source_edit_entry_batch {
    my ($cb, $app, $tmpl_ref) = @_;
    # th
    my $old = quotemeta(<<'HTML');
    <th class="col head title primary"><span class="col-label"><__trans phrase="Title"></span></th>
HTML
    my $new = <<HTML;
    <th class="col head stock"><span class="col-label"><__trans_section component="SkeletonCartConnector"><__trans phrase="Stock Quantity(Real/Adjust)"></__trans_section></span></th>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;

    # td
    $old = quotemeta(<<'HTML');
        <mt:var name="title" escape="html" default="...">
      </mt:if>
      </td>
HTML
    $new = <<HTML;
      <td class="col stock">
      <__trans_section component="SkeletonCartConnector">
      <mt:if name="item_available">
      <mt:if name="has_edit_access">
        <input type="text" name="item_stock_quantity_<mt:var name="id"/>" class="text num" value="<mt:var name="item_stock_quantity" escape="html" default="0" />" readonly="readonly" />
        <input type="text" name="item_stock_quantity_adjust_<mt:var name="id"/>" class="text num" value="<mt:var name="item_stock_quantity_adjust" escape="html" default="0" />" />
        <select name="item_stock_quantity_adjust_type_<mt:var name="id"/>">
          <option value="add"><__trans phrase="Add" /></option>
          <option value="reduce"><__trans phrase="Reduce" /></option>
        </select>
      <mt:else>
        <mt:var name="item_stock_quantity" escape="html" default="0" />
      </mt:if>
      <mt:else>
        -
      </mt:if>
      </__trans_section>
      </td>
HTML
    $$tmpl_ref =~ s!($old)!$1$new!;

    # for css
    $old = quotemeta(<<'HTML');
<mt:include name="include/header.tmpl" id="header_include">
HTML
    $new = <<HTML;
<mt:setvarblock name="html_head" append="1">
<style>
.batch-edit .listing-table td.stock input {
    width: 30%;
}
.batch-edit .listing-table td.stock select {
    width: 35%;
}
</style>
</mt:setvarblock>
HTML
    $$tmpl_ref =~ s!($old)!$new$1!;

    return 1;
}

sub param_edit_entry_batch {
    my ($cb, $app, $param, $tmpl) = @_;

    return unless enabled();
    return unless UNIVERSAL::isa($tmpl, 'MT::Template');

    require SkeletonCart::Item;
    my $i = 0;
    foreach my $row (@{$param->{object_loop}}) {
        my $entry_id = $row->{id};
        if($entry_id) {
            my ($item) = SkeletonCart::Item->load({ entry_id => $entry_id });
            if($item) {
                $param->{object_loop}->[$i]->{item_available} = 1;
                $param->{object_loop}->[$i]->{item_stock_quantity} = $item->stock_quantity;
            }
            else {
                $param->{object_loop}->[$i]->{item_available} = 0;
            }
        }
        $i++;
    }
}

sub cb_init_app {
    my ($cb, $app) = @_;

    require SkeletonCart::Item;
    require MT::Entry;
    no strict 'refs';
    no warnings 'redefine';
    *MT::Entry::sc_item = sub {
        SkeletonCart::Item::item_for_entry(@_);
    };
}

sub cb_init_request {
    my ($cb, $app) = @_;
    return 1 unless (is_mt51() || is_mt52() || is_mt6()) && enabled() && stock_enabled();
    my $plugin = MT->component('SkeletonCartConnector');
    $plugin->registry(
        list_properties => entry => item_stock_quantity => {
            base => '__virtual.integer',
            col => 'item_stock_quantity',
            label => $plugin->translate('ItemStockQuantity'),
            display => 'force',
            order => 210,
            html => sub {
                my $prop = shift;
                my ($obj, $app, $opts) = @_;
                my $item = $obj->sc_item
                    or return '';
                my $qty = $item->stock_quantity;
                return defined $qty ? $qty : '';
            }
        },
    );
    return 1;
}

sub cb_build_page {
    my ($cb, %args) = @_;
    my $map = $args{template_map}
        or return 1;
    require MT::PublishOption;
    return 1 unless ($map->build_type);
    return 1 if ($map->build_type == MT::PublishOption::DYNAMIC());
    return 1 unless (enabled() && stock_enabled() && use_dynamic_block());

    my $ctx = $args{context};
    my $entry_ids = $ctx->stash('skeletoncart_entry_ids') || []; 
    return 1 unless $entry_ids && @$entry_ids;

    my $app = MT->instance;
    my $cgi_server_path = $app->server_path() || '';
    $cgi_server_path =~ s!/*$!!;
    my $mt_php_path = File::Spec->canonpath("$cgi_server_path/php/mt.php");
    my $mt_util_path = File::Spec->canonpath("$cgi_server_path/php/lib/MTUtil.php");
    my $blog = $args{blog};
    my $blog_id = $blog->id;
    my $config_file = $app->{cfg_file};
    my $entry_ids_str = join ',', @$entry_ids; 
    my $create_instance = '';
    if(is_mt5_or_later()) {
        $create_instance = "\$mt = MT::get_instance($blog_id, '$config_file');";
    }
    else {
        $create_instance = "\$mt = new MT($blog_id, '$config_file');";
    }

    my $prefix = <<HTML;
<?php
include('$mt_php_path');
require_once('$mt_util_path');
$create_instance
\$mt->init_plugins();
\$ctx = \$mt->context();
\$entry_ids = array($entry_ids_str);
\$item_stock_quantity = skeletoncartconnector_get_item_real_stock_quantity(\$ctx, $blog_id, \$entry_ids);
?>
HTML

    my $content = $args{content};
    if($content && $$content) {
        $$content = $prefix . $$content;
    }
    return 1;
}

sub cb_save_config_filter {
    my ($cb, $plugin, $data, $scope) = @_;
    return 1;
}

1;
__END__
