package SkeletonCart::Tags;

use strict;
use warnings;
use utf8;
use Carp;
use MT;
use SkeletonCart;
use SkeletonCart::App;

sub _get_item_object_value {
    my ($ctx, $entry_id, $key) = @_;
    require SkeletonCart::Item;
    my $value = '';
    my $item = $ctx->stash('sc_item');
    unless($item && $item->entry_id == $entry_id) {
        ($item) = SkeletonCart::Item->load({ entry_id => $entry_id });
        $ctx->stash('sc_item', $item);
    }
    if($item) {
        my $column = $key eq 'id' ? 'item_id' : $key;
        $value = $item->column($column);
    }
    return $value;
}

sub _hdlr_id {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    # my $value = use_entry_id($entry->blog_id) ? $entry->id : $entry->item_id;
    my $value = '';
    my $use_entry_id = use_entry_id($entry->blog_id);
    if($use_entry_id) {
        $value = $entry->id;
    }
    else {
        $value = _get_item_object_value($ctx, $entry->id, 'id');
    }
    # return defined $value ? $value : '';
    return $value;
}

sub _get_item_value {
    my ($key, $ctx, $args) = @_;
    return '' unless $key;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    # my $value = $entry->column("item_$key");
    my $value = _get_item_object_value($ctx, $entry->id, $key);
    return defined $value ? $value : '';
}

sub _hdlr_name {
    return _get_item_value('name', @_);
}

sub _hdlr_price {
    return _get_item_value('price', @_);
}

sub _hdlr_stock {
    # NOTE: v2で廃止。sales_statusの値を返す。
    return _hdlr_sales_status(@_);
}

sub _hdlr_sales_status {
    return _get_item_value('sales_status', @_);
}

sub _hdlr_stock_message {
    return _get_item_value('stock_message', @_);
}

sub _hdlr_status_message {
    return _get_item_value('status_message', @_);
}

sub _hdlr_stock_quantity {
    return _get_item_value('stock_quantity', @_);
}

sub _hdlr_item_real_stock_quantity {
    my ( $ctx, $args ) = @_;
    my $blog_id = $ctx->stash('blog_id');
    my $use_dynamic_block = (enabled($blog_id) && stock_enabled($blog_id) && use_dynamic_block($blog_id));
    return '' unless $use_dynamic_block;

    my $entry = $ctx->stash('entry')
      or return $ctx->_no_entry_error();
    my $entry_ids = $ctx->stash('skeletoncart_entry_ids') || [];
    unless(grep { $_ eq $entry->id } @$entry_ids) {
        push @$entry_ids, $entry->id;
        $ctx->stash( 'skeletoncart_entry_ids', $entry_ids );
    }

    my $value = '$item_stock_quantity[' . $entry->id . ']';
    if($args->{numify}) {
        my $attr = $args->{numify};
        $value = "skeletoncartconnector_numify($value, $attr)";
    }

    my $output = '';
    if($args->{nophpwrap}) {
        $output = $value;
    }
    else {
        $output = '<?php echo(' . $value . '); ?>';
    }
    return $output;
}

sub _hdlr_if_item_real_stock_quantity {
    my ($ctx, $args, $cond) = @_;
    my $blog_id = $ctx->stash('blog_id');
    my $use_dynamic_block = (enabled($blog_id) && stock_enabled($blog_id) && use_dynamic_block($blog_id));
    return '' unless $use_dynamic_block;

    my $entry = $ctx->stash('entry')
      or return $ctx->_no_entry_error();
    my $entry_ids = $ctx->stash('skeletoncart_entry_ids') || [];
    unless(grep { $_ eq $entry->id } @$entry_ids) {
        push @$entry_ids, $entry->id;
        $ctx->stash( 'skeletoncart_entry_ids', $entry_ids );
    }

    my @ops = qw(eq ne gt lt ge le);
    my %op_map = (eq => '==', ne => '!=', gt => '>', lt => '<', ge => '>=', le => '<=');
    my $operator = '>';
    my $operand = 0;
    my $numeric = qr/^[-]?[0-9]+(\.[0-9]+)?$/;
    foreach my $op (@ops) {
        if(exists $args->{$op}) {
            $operand = $args->{$op};
            $operator = $op_map{$op};
            next;
        }
    }
    unless($operand =~ m/$numeric/) {
        $operator = '==';
    }

    my $if_sentense = '<?php if($item_stock_quantity['.$entry->id.'] ' . "$operator $operand" . ") { ?>\n";
    my $else_sentense = "<?php  } else { ?>\n";
    my $endif_sentense = "<?php } ?>\n";

    my $if_block = $ctx->slurp(@_);
    my $else_block = $ctx->else();
    my $out = $if_sentense . $if_block;
    if(defined $else_block) {
        $out .= $else_sentense . $else_block;
    }
    $out .= $endif_sentense;
    return $out;
}

sub _hdlr_group_id {
    return _get_item_value('group_id', @_);
}

sub _hdlr_scdir {
    my ($ctx, $args) = @_;
    my $blog_id = $ctx->stash('blog_id');
    my $dir = install_dir($blog_id);
    if($dir =~ m!/$!) {
        $dir =~ s!/$!!;
    }
    return defined $dir ? $dir : '';
}

sub _hdlr_weight {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $value = use_item_weight($entry->blog_id) ? _get_item_value('weight', @_) : '';
    return $value;
}

sub _hdlr_if_use_dynamic_block {
    my ($ctx, $args, $cond) = @_;
    my $blog_id = $ctx->stash('blog_id');
    my $use_dynamic_block = (enabled($blog_id) && stock_enabled($blog_id) && use_dynamic_block($blog_id));
    return $use_dynamic_block;
}

sub _hdlr_dynamic_block {
    my ($ctx, $args, $cond) = @_;
    my $blog_id = $ctx->stash('blog_id');
    my $use_dynamic_block = (enabled($blog_id) && stock_enabled($blog_id) && use_dynamic_block($blog_id));
    return '' unless $use_dynamic_block;
    my $vars = $ctx->{__stash}{vars};
    $vars->{skeletoncart_dynamic_block} = $use_dynamic_block;
    my $out = $ctx->slurp($args, $cond);
    return defined $out ? $out : '';
}

sub _make_option_field {
    my ($n, $ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    no strict 'refs';
    return '' unless $n
        && defined &{"use_item_option_field_$n"}
            && &{"use_item_option_field_$n"}($entry->blog_id);
    # my $str = $entry->column("item_option_field_$n");
    my $str = _get_item_object_value($ctx, $entry->id, "option_field_$n");
    my @list = split(/[\r\n]+/, $str);
    @list = grep { length $_ > 0 } @list;
    return '' unless @list;
    return '' unless defined &{"item_option_number_$n"};
    require MT::Util;
    my $number = &{"item_option_number_$n"}($entry->blog_id);
    my $select = '<select name="opt'. MT::Util::encode_html($number) .'">';
    my $option_format; 
    # NOTE: オプション価格を使用時 => qq{<mt:var name="name" /><mt:if name="price"> (+<mt:var name="price" />円)</mt:if>};
    my $format_string = qq{<mt:var name="name" />};
    if($args->{option_format}) {
        $option_format = $args->{option_format};
    }
    require MT::Template;
    my $tmpl = MT::Template->new(type => 'scalarref', source => \$format_string);
    if($option_format && ref $option_format eq 'MT::Template::Tokens') {
        $tmpl->tokens($option_format);
    }

    my $price_map = {};
    my $comma_price = $args->{comma_price} ? 1 : 0;
    foreach my $item (@list) {
        my ($name, $price) = map { MT::Util::trim($_) } split(/,/, $item);
        next unless length $name;
        $price = 0 unless $price;
        my $raw_price = $price;
        $price =~ s/(\d{1,3})(?=(?:\d{3})+(?!\d))/$1,/g if $comma_price;
        my $option_text = $tmpl->output({ name => $name, price => $price });
        $option_text = MT::Util::encode_html($option_text);
        $name = MT::Util::encode_html($name);
        $select .= '<option value="'.$name.'">'.$option_text.'</option>'."\n";
        $price_map->{$name} = int($raw_price);
    }
    $select .= '</select>';
    return $select;
}

sub _hdlr_option_field_1 {
    return _make_option_field(1, @_);
}

sub _hdlr_option_field_2 {
    return _make_option_field(2, @_);
}

sub _hdlr_option_field_3 {
    return _make_option_field(3, @_);
}

sub _hdlr_option_field_4 {
    return _make_option_field(4, @_);
}

sub _hdlr_option_field_5 {
    return _make_option_field(5, @_);
}

sub _hdlr_option_label_1 {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $label = item_option_label_1($entry->blog_id);
    return defined $label ? $label : '';
}

sub _hdlr_option_label_2 {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $label = item_option_label_2($entry->blog_id);
    return defined $label ? $label : '';
}

sub _hdlr_option_label_3 {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $label = item_option_label_3($entry->blog_id);
    return defined $label ? $label : '';
}

sub _hdlr_option_label_4 {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $label = item_option_label_4($entry->blog_id);
    return defined $label ? $label : '';
}

sub _hdlr_option_label_5 {
    my ($ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    my $label = item_option_label_5($entry->blog_id);
    return defined $label ? $label : '';
}

sub _get_option_price {
    my ($n, $ctx, $args) = @_;
    my $entry = $ctx->stash('entry')
        or return $ctx->_no_entry_error();
    no strict 'refs';
    return {} unless $n
        && defined &{"use_item_option_field_$n"}
            && &{"use_item_option_field_$n"}($entry->blog_id);
    # my $str = $entry->column("item_option_field_$n");
    my $str = _get_item_object_value($ctx, $entry->id, "option_field_$n");
    my @list = split(/[\r\n]+/, $str);
    @list = grep { length $_ > 0 } @list;
    return {} unless @list;
    return {} unless defined &{"item_option_number_$n"};

    require MT::Util;
    my $price_map = {};
    foreach my $item (@list) {
        my ($name, $price) = map { MT::Util::trim($_) } split(/,/, $item);
        next unless length $name;
        $price = 0 unless $price;
        $price_map->{$name} = int($price);
    }

    return $price_map;
}

sub _to_json {
    my ($obj, $canonical) = @_;
    require JSON;
    my $json = JSON->new->utf8(0);
    if($canonical) {
        $json = $json->canonical;
    }
    my $json_str = $json->encode($obj);
    return $json_str;
}

sub _hdlr_support_script {
    my ($ctx, $args) = @_;

    my $obj = {};
    my $item_price = _get_item_value('price', $ctx, $args);
    $obj->{price} = int($item_price);

    for(1..5) {
        $obj->{'opt' . $_} = _get_option_price($_, $ctx, $args);
    }

    my $json_str = _to_json($obj, $args->{canonical});
    my $script = <<SCRIPT;
<script type="text/javascript">
if(typeof SkeletonCart === "undefined") {
    SkeletonCart = {};
}
SkeletonCart = $json_str;

var updateTotalPrice = function() {
    var \$total = jQuery('span.total_price');
    if(\$total.length == 0) {
        return;
    }
    var total = SkeletonCart.price || 0;
    for(var n=1; n<=5; n++) {
        var \$selected = jQuery('select[name="opt' + n + '"] :selected');
        if(\$selected.length == 0) {
            continue;
        }
        var opt_name = \$selected.val();
        if(!opt_name) {
            continue;
        }
        var opt_price = SkeletonCart['opt'+n][opt_name] || 0;
        if(opt_price > 0) {
            total += opt_price;
        }
    }
    if(total > 0 && \$total.data('apply-amount')) {
        var amount = 0;
        var \$amount = jQuery('select[name="amount"] :selected');
        if(\$amount.length > 0) {
            amount = parseInt(\$amount.val());
            if(isNaN(amount)) {
                amount = 0;
            }
            if(amount > 0) {
                total *= amount;
            }
        }
    }
    if(total > 0 && \$total.data('comma-price')) {
        total = total.toString().replace(/(\\d)(?=(\\d\\d\\d)+(?!\\d))/g, '\$1,');
    }
    \$total.text(total);
}
jQuery('select[name^="opt"], select[name="amount"]').change(function() {
    updateTotalPrice();
});
jQuery(function() {
    updateTotalPrice();
});
</script>
SCRIPT

    return $script;
}

1;
__END__


