<?php
global $mt;
$ctx =& $mt->context();
$ctx->register_prefilter('prefilter_scdynamicblock');
$ctx->register_prefilter('prefilter_ifitemrealstockquantity');

function skeletoncartconnector_get_plugin_config(&$ctx) {
    $blog = $ctx->stash('blog');
    $config = $ctx->mt->db()->fetch_plugin_config('SkeletonCartConnector', 'blog:'.$blog->blog_id);
    return $config;
} 

function skeletoncartconnector_get_plugin_config_value(&$ctx, $key) {
    #$mt = MT::get_instance();
    $blog = $ctx->stash('blog');

    $config = skeletoncartconnector_get_plugin_config($ctx);
    $value = '';
    if (isset($config[$key])) {
       $value = $config[$key]; 
    }
    return $value;
}

function skeletoncartconnector_make_option_field($args, &$ctx, $n) {
    if(!$n) {
        return '';
    }
    $blog = $ctx->stash('blog');
    $entry = $ctx->stash('entry');
    if(!$entry) {
        return '';
    }
    $item = skeletoncartconnector_get_item($ctx, $entry->id);
    if(!$item) {
        return '';
    }

    $config = skeletoncartconnector_get_plugin_config($ctx);
    if(!isset($config['use_item_option_field_'.$n])) {
        return '';
    }

    $method = 'option_field_'.$n;
    $str = $item->$method;
    $list = preg_split('/[\r\n]+/', $str, -1, PREG_SPLIT_NO_EMPTY);
    if(count($list) == 0) {
        return '';
    }
    $number = $config['item_option_number_'.$n];
    if(!$number) {
        return '';
    }
    $select = '<select name="opt' . htmlspecialchars($number) . '">';
    $format = $args['option_format'];
    if(empty($format)) {
        $format = _get_default_format($ctx);
    }
    $comma_price = $args['comma_price'] ? 1 : 0;
    $option_prices = array();
    foreach($list as $item) {
        $items = explode(',', $item);
        $name = $items[0];
        $price = $items[1];
        if(!$price) {
            $price = 0;
        }
        $option_prices[$name] = $price;
        if($comma_price) {
            $price = number_format($price);
        }
        $option_text = _format_option_text($ctx, $format, array('name'=>$name, 'price'=>$price));
        $option_text = htmlspecialchars($option_text);
        $name = htmlspecialchars($name);
        $select .= '<option value="'.$name.'">'.$option_text.'</option>'."\n";
    }
    $select .= '</select>';
    return $select;
}

/* NOTE: オプション価格を利用する場合には以下のフォーマットを使用する。
   <mt:var name="name" /><mt:if name="price"> (+<mt:var name="price" />円)</mt:if>
 */
function _get_default_format(&$ctx) {
    $source = <<<TMPL
<mt:setvartemplate name="__sc_format_option_text">
<mt:var name="name" />
</mt:setvartemplate>
TMPL;
    $_var_compiled = '';
    if(!$ctx->_compile_source('evaluated template', $source, $_var_compiled)) {
        return $ctx->error('Error compiling template source');
    }
    ob_start();
    $ctx->_eval('?>' . $_var_compiled);
    $_contents = ob_get_contents();
    ob_end_clean();
    $format = $ctx->__stash['vars']['__sc_format_option_text'];
    return $format;
}

function _format_option_text(&$ctx, $format, $params) {
    $return_val = '';
    if(!is_array($format) && preg_match('/^smarty_fun_[a-f0-9]+$/', $format)) {
        if(function_exists($format)) {
            $localvars = array(null, array_keys($params));
            $ctx->localize($localvars);
            foreach($params as $key => $value) {
                $ctx->__stash['vars'][$key] = $value;
            }
            ob_start();
            $format($ctx, array());
            $return_val = ob_get_contents();
            ob_end_clean();
            $ctx->restore($localvars);
        }
    }
    return $return_val;
}

function prefilter_scdynamicblock($tpl_source, $ctx2) {
    return preg_replace('/<MT:?ScDynamicBlock[^>]*?>.*?<\/MT:?ScDynamicBlock>/is', '', $tpl_source);
} 

function prefilter_ifitemrealstockquantity($tpl_source, $ctx2) {
    return preg_replace('/<MT:?IfItemRealStockQuantity[^>]*?>.*?<\/MT:?IfItemRealStockQuantity>/is', '', $tpl_source);
}

function skeletoncartconnector_get_item_real_stock_quantity(&$ctx, $blog_id, $entry_ids) {
    $mtdb =& $ctx->mt->db();
    $quantities = array();
    if(empty($entry_ids)) {
        return $quantities;
    }
    $blog = $mtdb->fetch_blog($blog_id);
    $ctx->stash('blog', $blog);
    $ctx->stash('blog_id', $blog_id);
    $entry_ids_str = implode(',', $entry_ids);
    $args = array('not_entry_id' => "0 and entry_id IN ( $entry_ids_str )");
    $args['limit'] = count($entry_ids);
    $entries = $mtdb->fetch_entries($args);
    if(!empty($entries)) {
        foreach($entries as $entry) {
            $item = skeletoncartconnector_get_item($ctx, $entry->id);
            if(!$item) {
                continue;
            }
            $quantity = $item->stock_quantity;
            if(empty($quantity)) {
                $quantity = 0;
            }
            $quantities[$entry->entry_id] = $quantity;
        }
    }
    return $quantities;
}

function _get_option_price($n, $args, &$ctx) {
    if(!$n) {
        return (object)array();
    }
    $blog = $ctx->stash('blog');
    $entry = $ctx->stash('entry');
    if(!$entry) {
        return (object)array();
    }
    $item = skeletoncartconnector_get_item($ctx, $entry->id);
    if(!$item) {
        return (object)array();
    }

    $config = skeletoncartconnector_get_plugin_config($ctx);
    if(!isset($config['use_item_option_field_'.$n])) {
        return (object)array();
    }

    $method = 'option_field_'.$n;
    $str = $item->$method;
    $list = preg_split('/[\r\n]+/', $str, -1, PREG_SPLIT_NO_EMPTY);
    if(count($list) == 0) {
        return (object)array();
    }
    $number = $config['item_option_number_'.$n];
    if(!$number) {
        return (object)array();
    }

    $price_map = array(); 
    foreach($list as $item) {
        $items = explode(',', $item);
        $name = $items[0];
        if(!$name) {
            continue;
        }
        $price = $items[1];
        if(!$price) {
            $price = 0;
        }
        $price_map[$name] = intval($price);
    }
    return $price_map;
}

function skeletoncartconnector_support_script($args, &$ctx) {
    $obj = array();
    $item_price = smarty_function_mtitemprice($args, $ctx);
    $obj['price'] = intval($item_price);

    for($i=1; $i<=5; $i++) {
        $obj['opt' . $i] = _get_option_price($i, $args, $ctx);
    }
    $json_obj = json_encode($obj);

    $script = <<<SCRIPT
<script type="text/javascript">
if(typeof SkeletonCart === "undefined") {
    SkeletonCart = {};
}
SkeletonCart = $json_obj;

var updateTotalPrice = function() {
    var \$total = jQuery('span.total_price');
    if(\$total.length == 0) {
        return;
    }
    var total = SkeletonCart.price || 0;
    for(var n=1; n<=5; n++) {
        var \$selected = jQuery('select[name="opt' + n + '"] :selected');
        if(\$selected.length == 0) {
            continue;
        }
        var opt_name = \$selected.val();
        if(!opt_name) {
            continue;
        }
        var opt_price = SkeletonCart['opt'+n][opt_name] || 0;
        if(opt_price > 0) {
            total += opt_price;   
        }
    }
    if(total > 0 && \$total.data('apply-amount')) {
        var amount = 0;
        var \$amount = jQuery('select[name="amount"] :selected');
        if(\$amount.length > 0) {
            amount = parseInt(\$amount.val());
            if(isNaN(amount)) {
                amount = 0;
            }
            if(amount > 0) {
                total *= amount;
            }
        }
    }
    if(total > 0 && \$total.data('comma-price')) {
        total = total.toString().replace(/(\\d)(?=(\\d\\d\\d)+(?!\\d))/g, '\$1,');
    }
    \$total.text(total);
}
jQuery('select[name^="opt"], select[name="amount"]').change(function() {
    updateTotalPrice();
});
jQuery(function() {
    updateTotalPrice();
});
</script>
SCRIPT;

    return $script;
}

function skeletoncartconnector_fetch_item($entry_id) {
    $where = "sc_item_entry_id=$entry_id";
    require_once('class.mt_sc_item.php');
    $item = new ScItem;
    $items = $item->Find($where);
    if(!empty($items)) {
        $item = $items[0];
    }
    else {
        $item = null;
    }
    return $item;
}

function skeletoncartconnector_get_item(&$ctx, $entry_id) {
    $item = $ctx->stash('sc_item');
    if(!$item || $item->entry_id !== $entry_id) {
        $item = skeletoncartconnector_fetch_item($entry_id);
        if($item) {
            $ctx->stash('sc_item', $item);
        }
    }
    return $item;
}

function skeletoncartconnector_numify($text, $attr = ",") {
    if(!function_exists('smarty_modifier_numify')) {
        if ($attr == "1") $attr = ',';
        $attr = str_replace(array('\\', '$'), array('\\\\', '\$'), $attr);
        return preg_replace('/(^[-+]?\d+?(?=(?>(?:\d{3})+)(?!\d))|\G\d{3}(?=\d))/', '${1}' . $attr, $text);
    }
    return smarty_modifier_numify($text, $attr);
}

?>
